@@ -14,14 +14,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from datetime import datetime
+import datetime
 
+from oslo.utils import strutils
 import six
 from six.moves.urllib import parse
 
 from novaclient import client as base_client
 from novaclient import exceptions
-from novaclient.openstack.common import strutils
 from novaclient.tests import fakes
 from novaclient.tests import utils
 from novaclient.v1_1 import client
@@ -369,7 +369,10 @@ class FakeHTTPClient(base_client.HTTPClient):
         if 'personality' in body['server']:
             for pfile in body['server']['personality']:
                 fakes.assert_has_keys(pfile, required=['path', 'contents'])
-        return (202, {}, self.get_servers_1234()[2])
+        if body['server']['name'] == 'some-bad-server':
+            return (202, {}, self.get_servers_1235()[2])
+        else:
+            return (202, {}, self.get_servers_1234()[2])
 
     def post_os_volumes_boot(self, body, **kw):
         assert set(body.keys()) <= set(['server', 'os:scheduler_hints'])
@@ -392,6 +395,13 @@ class FakeHTTPClient(base_client.HTTPClient):
         r = {'server': self.get_servers_detail()[2]['servers'][0]}
         return (200, {}, r)
 
+    def get_servers_1235(self, **kw):
+        r = {'server': self.get_servers_detail()[2]['servers'][0]}
+        r['server']['id'] = 1235
+        r['server']['status'] = 'error'
+        r['server']['fault'] = {'message': 'something went wrong!'}
+        return (200, {}, r)
+
     def get_servers_5678(self, **kw):
         r = {'server': self.get_servers_detail()[2]['servers'][1]}
         return (200, {}, r)
@@ -405,6 +415,12 @@ class FakeHTTPClient(base_client.HTTPClient):
         fakes.assert_has_keys(body['server'], optional=['name', 'adminPass'])
         return (204, {}, body)
 
+    def delete_os_server_groups_12345(self, **kw):
+        return (202, {}, None)
+
+    def delete_os_server_groups_56789(self, **kw):
+        return (202, {}, None)
+
     def delete_servers_1234(self, **kw):
         return (202, {}, None)
 
@@ -525,11 +541,11 @@ class FakeHTTPClient(base_client.HTTPClient):
             assert list(body[action]) == ['type']
             assert body[action]['type'] in ['HARD', 'SOFT']
         elif action == 'rebuild':
-            keys = list(body[action])
-            if 'adminPass' in keys:
-                keys.remove('adminPass')
-            assert 'imageRef' in keys
+            body = body[action]
+            adminPass = body.get('adminPass', 'randompassword')
+            assert 'imageRef' in body
             _body = self.get_servers_1234()[2]
+            _body['server']['adminPass'] = adminPass
         elif action == 'resize':
             keys = body[action].keys()
             assert 'flavorRef' in keys
@@ -620,11 +636,16 @@ class FakeHTTPClient(base_client.HTTPClient):
             keys = list(body[action])
             if 'adminPass' in keys:
                 keys.remove('adminPass')
-            assert set(keys) == set(['host', 'onSharedStorage'])
+            if 'host' in keys:
+                keys.remove('host')
+            assert set(keys) == set(['onSharedStorage'])
         else:
             raise AssertionError("Unexpected server action: %s" % action)
         return (resp, _headers, _body)
 
+    def post_servers_5678_action(self, body, **kw):
+        return self.post_servers_1234_action(body, **kw)
+
     #
     # Cloudpipe
     #
@@ -691,14 +712,14 @@ class FakeHTTPClient(base_client.HTTPClient):
         if filter_is_public is not None:
             if filter_is_public:
                 flavors['flavors'] = [
-                        v for v in flavors['flavors']
-                            if v['os-flavor-access:is_public']
-                        ]
+                    v for v in flavors['flavors']
+                    if v['os-flavor-access:is_public']
+                ]
             else:
                 flavors['flavors'] = [
-                        v for v in flavors['flavors']
-                            if not v['os-flavor-access:is_public']
-                        ]
+                    v for v in flavors['flavors']
+                    if not v['os-flavor-access:is_public']
+                ]
 
         return (200, {}, flavors)
 
@@ -734,8 +755,11 @@ class FakeHTTPClient(base_client.HTTPClient):
     def get_flavors_512_MB_Server(self, **kw):
         raise exceptions.NotFound('404')
 
+    def get_flavors_128_MB_Server(self, **kw):
+        raise exceptions.NotFound('404')
+
     def get_flavors_aa1(self, **kw):
-        # Aplhanumeric flavor id are allowed.
+        # Alphanumeric flavor id are allowed.
         return (
             200,
             {},
@@ -1003,12 +1027,24 @@ class FakeHTTPClient(base_client.HTTPClient):
     # Keypairs
     #
     def get_os_keypairs_test(self, *kw):
-        return (200, {}, {'keypair': self.get_os_keypairs()[2]['keypairs'][0]})
+        return (200, {}, {'keypair':
+                          self.get_os_keypairs()[2]['keypairs'][0]['keypair']})
 
     def get_os_keypairs(self, *kw):
         return (200, {}, {"keypairs": [
-            {'fingerprint': 'FAKE_KEYPAIR', 'name': 'test'}
-        ]})
+                              {"keypair": {
+                                  "public_key": "FAKE_SSH_RSA",
+                                  "private_key": "FAKE_PRIVATE_KEY",
+                                  "user_id":
+                                      "81e373b596d6466e99c4896826abaa46",
+                                  "name": "test",
+                                  "deleted": False,
+                                  "created_at": "2014-04-19T02:16:44.000000",
+                                  "updated_at": "2014-04-19T10:12:3.000000",
+                                  "figerprint": "FAKE_KEYPAIR",
+                                  "deleted_at": None,
+                                  "id": 4}
+                               }]})
 
     def delete_os_keypairs_test(self, **kw):
         return (202, {}, None)
@@ -1017,7 +1053,7 @@ class FakeHTTPClient(base_client.HTTPClient):
         assert list(body) == ['keypair']
         fakes.assert_has_keys(body['keypair'],
                               required=['name'])
-        r = {'keypair': self.get_os_keypairs()[2]['keypairs'][0]}
+        r = {'keypair': self.get_os_keypairs()[2]['keypairs'][0]['keypair']}
         return (202, {}, r)
 
     #
@@ -1277,7 +1313,7 @@ class FakeHTTPClient(base_client.HTTPClient):
     def get_os_security_group_rules(self, **kw):
         return (200, {}, {"security_group_rules": [
                 {'id': 1, 'parent_group_id': 1, 'group_id': 2,
-                 'ip_protocol': 'TCP', 'from_port': '22', 'to_port': 22,
+                 'ip_protocol': 'TCP', 'from_port': 22, 'to_port': 22,
                  'cidr': '10.0.0.0/8'}
         ]})
 
@@ -1300,6 +1336,34 @@ class FakeHTTPClient(base_client.HTTPClient):
             self.get_os_security_group_rules()[2]['security_group_rules'][0]}
         return (202, {}, r)
 
+    #
+    # Security Group Default Rules
+    #
+    def get_os_security_group_default_rules(self, **kw):
+        return (200, {}, {"security_group_default_rules": [
+                {'id': 1, 'ip_protocol': 'TCP', 'from_port': 22,
+                 'to_port': 22, 'cidr': '10.0.0.0/8'}
+        ]})
+
+    def delete_os_security_group_default_rules_1(self, **kw):
+        return (202, {}, None)
+
+    def delete_os_security_group_default_rules_11(self, **kw):
+        return (202, {}, None)
+
+    def delete_os_security_group_default_rules_12(self, **kw):
+        return (202, {}, None)
+
+    def post_os_security_group_default_rules(self, body, **kw):
+        assert list(body) == ['security_group_default_rule']
+        fakes.assert_has_keys(body['security_group_default_rule'],
+            optional=['ip_protocol', 'from_port',
+                      'to_port', 'cidr'])
+        rules = self.get_os_security_group_default_rules()
+        r = {'security_group_default_rule':
+            rules[2]['security_group_default_rules'][0]}
+        return (202, {}, r)
+
     #
     # Tenant Usage
     #
@@ -1310,7 +1374,7 @@ class FakeHTTPClient(base_client.HTTPClient):
                     six.u('total_vcpus_usage'): 49.71047423333333,
                     six.u('total_hours'): 49.71047423333333,
                     six.u('tenant_id'):
-                         six.u('7b0a1d73f8fb41718f3343c207597869'),
+                        six.u('7b0a1d73f8fb41718f3343c207597869'),
                     six.u('stop'): six.u('2012-01-22 19:48:41.750722'),
                     six.u('server_usages'): [{
                         six.u('hours'): 49.71047423333333,
@@ -1319,7 +1383,7 @@ class FakeHTTPClient(base_client.HTTPClient):
                         six.u('ended_at'): None,
                         six.u('name'): six.u('f15image1'),
                         six.u('tenant_id'):
-                             six.u('7b0a1d73f8fb41718f3343c207597869'),
+                            six.u('7b0a1d73f8fb41718f3343c207597869'),
                         six.u('vcpus'): 1,
                         six.u('memory_mb'): 512,
                         six.u('state'): six.u('active'),
@@ -1336,7 +1400,7 @@ class FakeHTTPClient(base_client.HTTPClient):
                     six.u('total_vcpus_usage'): 49.71047423333333,
                     six.u('total_hours'): 49.71047423333333,
                     six.u('tenant_id'):
-                         six.u('7b0a1d73f8fb41718f3343c207597869'),
+                        six.u('7b0a1d73f8fb41718f3343c207597869'),
                     six.u('stop'): six.u('2012-01-22 19:48:41.750722'),
                     six.u('server_usages'): [{
                         six.u('hours'): 49.71047423333333,
@@ -1422,15 +1486,25 @@ class FakeHTTPClient(base_client.HTTPClient):
             {'id': '2',
              'name': 'test2',
              'availability_zone': 'nova1'},
+            {'id': '3',
+             'name': 'test3',
+             'metadata': {'test': "dup", "none_key": "Nine"}},
         ]})
 
     def _return_aggregate(self):
         r = {'aggregate': self.get_os_aggregates()[2]['aggregates'][0]}
         return (200, {}, r)
 
+    def _return_aggregate_3(self):
+        r = {'aggregate': self.get_os_aggregates()[2]['aggregates'][2]}
+        return (200, {}, r)
+
     def get_os_aggregates_1(self, **kw):
         return self._return_aggregate()
 
+    def get_os_aggregates_3(self, **kw):
+        return self._return_aggregate_3()
+
     def post_os_aggregates(self, body, **kw):
         return self._return_aggregate()
 
@@ -1440,12 +1514,18 @@ class FakeHTTPClient(base_client.HTTPClient):
     def put_os_aggregates_2(self, body, **kw):
         return self._return_aggregate()
 
+    def put_os_aggregates_3(self, body, **kw):
+        return self._return_aggregate_3()
+
     def post_os_aggregates_1_action(self, body, **kw):
         return self._return_aggregate()
 
     def post_os_aggregates_2_action(self, body, **kw):
         return self._return_aggregate()
 
+    def post_os_aggregates_3_action(self, body, **kw):
+        return self._return_aggregate_3()
+
     def delete_os_aggregates_1(self, **kw):
         return (202, {}, None)
 
@@ -1461,13 +1541,17 @@ class FakeHTTPClient(base_client.HTTPClient):
                        'zone': 'nova',
                        'status': 'enabled',
                        'state': 'up',
-                       'updated_at': datetime(2012, 10, 29, 13, 42, 2)},
+                       'updated_at': datetime.datetime(
+                           2012, 10, 29, 13, 42, 2
+                       )},
                       {'binary': binary,
                        'host': host,
                        'zone': 'nova',
                        'status': 'disabled',
                        'state': 'down',
-                       'updated_at': datetime(2012, 9, 18, 8, 3, 38)},
+                       'updated_at': datetime.datetime(
+                           2012, 9, 18, 8, 3, 38
+                       )},
                       ]})
 
     def put_os_services_enable(self, body, **kw):
@@ -1486,6 +1570,9 @@ class FakeHTTPClient(base_client.HTTPClient):
                           'status': 'disabled',
                           'disabled_reason': body['disabled_reason']}})
 
+    def delete_os_services_1(self, **kw):
+        return (204, {}, None)
+
     #
     # Fixed IPs
     #
@@ -1671,13 +1758,17 @@ class FakeHTTPClient(base_client.HTTPClient):
     def get_os_networks(self, **kw):
         return (200, {}, {'networks': [{"label": "1", "cidr": "10.0.0.0/24",
                 'project_id': '4ffc664c198e435e9853f2538fbcd7a7',
-                'id': '1'}]})
+                'id': '1', 'vlan': '1234'}]})
+
+    def delete_os_networks_1(self, **kw):
+        return (202, {}, None)
 
     def post_os_networks(self, **kw):
         return (202, {}, {'network': kw})
 
     def get_os_networks_1(self, **kw):
-        return (200, {}, {'network': {"label": "1", "cidr": "10.0.0.0/24"}})
+        return (200, {}, {'network': {"label": "1", "cidr": "10.0.0.0/24",
+                                      "id": "1"}})
 
     def delete_os_networks_networkdelete(self, **kw):
         return (202, {}, None)
@@ -1744,7 +1835,9 @@ class FakeHTTPClient(base_client.HTTPClient):
                                        "nova-compute": {"active": True,
                                            "available": True,
                                            "updated_at":
-                                   datetime(2012, 12, 26, 14, 45, 25, 0)}}}},
+                                   datetime.datetime(
+                                       2012, 12, 26, 14, 45, 25, 0
+                                   )}}}},
                               {"zoneName": "internal",
                                "zoneState": {"available": True},
                                "hosts": {
@@ -1753,13 +1846,17 @@ class FakeHTTPClient(base_client.HTTPClient):
                                            "active": True,
                                            "available": True,
                                            "updated_at":
-                                   datetime(2012, 12, 26, 14, 45, 25, 0)}},
+                                   datetime.datetime(
+                                       2012, 12, 26, 14, 45, 25, 0
+                                   )}},
                                    "fake_host-2": {
                                        "nova-network": {
                                            "active": True,
                                            "available": False,
                                            "updated_at":
-                                   datetime(2012, 12, 26, 14, 45, 24, 0)}}}},
+                                   datetime.datetime(
+                                       2012, 12, 26, 14, 45, 24, 0
+                                   )}}}},
                               {"zoneName": "zone-2",
                                "zoneState": {"available": False},
                                "hosts": None}]})
@@ -1996,3 +2093,48 @@ class FakeHTTPClient(base_client.HTTPClient):
         return (200, {}, {'events': [
                     {'name': 'network-changed',
                      'server_uuid': '1234'}]})
+
+    #
+    # Server Groups
+    #
+
+    def get_os_server_groups(self, *kw):
+        return (200, {},
+                {"server_groups": [
+                 {"members": [], "metadata": {},
+                  "id": "2cbd51f4-fafe-4cdb-801b-cf913a6f288b",
+                  "policies": [], "name": "ig1"},
+                 {"members": [], "metadata": {},
+                  "id": "4473bb03-4370-4bfb-80d3-dc8cffc47d94",
+                  "policies": ["anti-affinity"], "name": "ig2"},
+                 {"members": [], "metadata": {"key": "value"},
+                  "id": "31ab9bdb-55e1-4ac3-b094-97eeb1b65cc4",
+                  "policies": [], "name": "ig3"},
+                 {"members": ["2dccb4a1-02b9-482a-aa23-5799490d6f5d"],
+                  "metadata": {},
+                  "id": "4890bb03-7070-45fb-8453-d34556c87d94",
+                  "policies": ["anti-affinity"], "name": "ig2"}]})
+
+    def _return_server_group(self):
+        r = {'server_group':
+             self.get_os_server_groups()[2]['server_groups'][0]}
+        return (200, {}, r)
+
+    def post_os_server_groups(self, body, **kw):
+        return self._return_server_group()
+
+    def get_os_server_groups_2cbd51f4_fafe_4cdb_801b_cf913a6f288b(self,
+                                                                    **kw):
+        return self._return_server_group()
+
+    def put_os_server_groups_2cbd51f4_fafe_4cdb_801b_cf913a6f288b(self,
+                                                                    **kw):
+        return self._return_server_group()
+
+    def post_os_server_groups_2cbd51f4_fafe_4cdb_801b_cf913a6f288b_action(
+            self, body, **kw):
+        return self._return_server_group()
+
+    def delete_os_server_groups_2cbd51f4_fafe_4cdb_801b_cf913a6f288b(
+            self, **kw):
+        return (202, {}, None)
