@@ -22,6 +22,7 @@ import os
 
 import fixtures
 import mock
+from oslo.utils import timeutils
 import six
 from six.moves import builtins
 
@@ -266,7 +267,10 @@ class ShellTest(utils.TestCase):
                     {
                         'volume_id': 'blah',
                         'delete_on_termination': '0',
-                        'device_name': 'vda'
+                        'device_name': 'vda',
+                        'uuid': 'blah',
+                        'boot_index': 0,
+                        'source_type': ''
                     }
                 ],
                 'imageRef': '',
@@ -886,6 +890,19 @@ class ShellTest(utils.TestCase):
         self.assert_called('GET', '/flavors/1', pos=-2)
         self.assert_called('GET', '/images/2')
 
+    def test_rebuild_name_meta(self):
+        self.run_command('rebuild sample-server 1 --name asdf --meta '
+                         'foo=bar')
+        self.assert_called('GET', '/servers', pos=-6)
+        self.assert_called('GET', '/servers/1234', pos=-5)
+        self.assert_called('GET', '/images/1', pos=-4)
+        self.assert_called('POST', '/servers/1234/action',
+                           {'rebuild': {'imageRef': 1,
+                                        'name': 'asdf',
+                                        'metadata': {'foo': 'bar'}}}, pos=-3)
+        self.assert_called('GET', '/flavors/1', pos=-2)
+        self.assert_called('GET', '/images/2')
+
     def test_start(self):
         self.run_command('start sample-server')
         self.assert_called('POST', '/servers/1234/action', {'os-start': None})
@@ -990,7 +1007,7 @@ class ShellTest(utils.TestCase):
         self.assert_called('GET', '/flavors/1', pos=-1)
 
     def test_show_bad_id(self):
-        self.assertRaises(exceptions.ResourceNotFound,
+        self.assertRaises(exceptions.CommandError,
                           self.run_command, 'show xxx')
 
     @mock.patch('novaclient.v1_1.shell.utils.print_dict')
@@ -1034,9 +1051,11 @@ class ShellTest(utils.TestCase):
         self.assert_called('DELETE', '/servers/5678', pos=-1)
 
     def test_delete_two_with_one_nonexistent(self):
-        self.run_command('delete 1234 123456789')
+        cmd = 'delete 1234 123456789'
+        self.assertRaises(exceptions.CommandError, self.run_command, cmd)
         self.assert_called_anytime('DELETE', '/servers/1234')
-        self.run_command('delete sample-server nonexistentserver')
+        cmd = 'delete sample-server nonexistentserver'
+        self.assertRaises(exceptions.CommandError, self.run_command, cmd)
         self.assert_called_anytime('DELETE', '/servers/1234')
 
     def test_delete_one_with_one_nonexistent(self):
@@ -1214,9 +1233,9 @@ class ShellTest(utils.TestCase):
                            'end=2005-02-01T00:00:00&' +
                            'detailed=1')
 
-    @mock.patch('novaclient.openstack.common.timeutils.utcnow')
-    def test_usage_list_no_args(self, mock_utcnow):
-        mock_utcnow.return_value = datetime.datetime(2005, 2, 1, 0, 0)
+    def test_usage_list_no_args(self):
+        timeutils.set_time_override(datetime.datetime(2005, 2, 1, 0, 0))
+        self.addCleanup(timeutils.clear_time_override)
         self.run_command('usage-list')
         self.assert_called('GET',
                            '/os-simple-tenant-usage?' +
@@ -1293,17 +1312,29 @@ class ShellTest(utils.TestCase):
         self.assert_called('PUT', '/os-aggregates/1', body, pos=-2)
         self.assert_called('GET', '/os-aggregates/1', pos=-1)
 
-    def test_aggregate_set_metadata_by_id(self):
-        self.run_command('aggregate-set-metadata 1 foo=bar delete_key')
-        body = {"set_metadata": {"metadata": {"foo": "bar",
-                                              "delete_key": None}}}
-        self.assert_called('POST', '/os-aggregates/1/action', body, pos=-2)
-        self.assert_called('GET', '/os-aggregates/1', pos=-1)
+    def test_aggregate_set_metadata_add_by_id(self):
+        self.run_command('aggregate-set-metadata 3 foo=bar')
+        body = {"set_metadata": {"metadata": {"foo": "bar"}}}
+        self.assert_called('POST', '/os-aggregates/3/action', body, pos=-2)
+        self.assert_called('GET', '/os-aggregates/3', pos=-1)
+
+    def test_aggregate_set_metadata_add_duplicate_by_id(self):
+        cmd = 'aggregate-set-metadata 3 test=dup'
+        self.assertRaises(exceptions.CommandError, self.run_command, cmd)
+
+    def test_aggregate_set_metadata_delete_by_id(self):
+        self.run_command('aggregate-set-metadata 3 none_key')
+        body = {"set_metadata": {"metadata": {"none_key": None}}}
+        self.assert_called('POST', '/os-aggregates/3/action', body, pos=-2)
+        self.assert_called('GET', '/os-aggregates/3', pos=-1)
+
+    def test_aggregate_set_metadata_delete_missing_by_id(self):
+        cmd = 'aggregate-set-metadata 3 delete_key2'
+        self.assertRaises(exceptions.CommandError, self.run_command, cmd)
 
     def test_aggregate_set_metadata_by_name(self):
-        self.run_command('aggregate-set-metadata test foo=bar delete_key')
-        body = {"set_metadata": {"metadata": {"foo": "bar",
-                                              "delete_key": None}}}
+        self.run_command('aggregate-set-metadata test foo=bar')
+        body = {"set_metadata": {"metadata": {"foo": "bar"}}}
         self.assert_called('POST', '/os-aggregates/1/action', body, pos=-2)
         self.assert_called('GET', '/os-aggregates/1', pos=-1)
 
@@ -1511,17 +1542,13 @@ class ShellTest(utils.TestCase):
         self.run_command('host-evacuate --on-shared-storage hyper')
         self.assert_called('GET', '/os-hypervisors/hyper/servers', pos=0)
         self.assert_called('POST', '/servers/uuid1/action',
-                           {'evacuate': {'host': None,
-                                         'onSharedStorage': True}}, pos=1)
+                           {'evacuate': {'onSharedStorage': True}}, pos=1)
         self.assert_called('POST', '/servers/uuid2/action',
-                           {'evacuate': {'host': None,
-                                         'onSharedStorage': True}}, pos=2)
+                           {'evacuate': {'onSharedStorage': True}}, pos=2)
         self.assert_called('POST', '/servers/uuid3/action',
-                           {'evacuate': {'host': None,
-                                         'onSharedStorage': True}}, pos=3)
+                           {'evacuate': {'onSharedStorage': True}}, pos=3)
         self.assert_called('POST', '/servers/uuid4/action',
-                           {'evacuate': {'host': None,
-                                         'onSharedStorage': True}}, pos=4)
+                           {'evacuate': {'onSharedStorage': True}}, pos=4)
 
     def test_host_servers_migrate(self):
         self.run_command('host-servers-migrate hyper')
@@ -1672,6 +1699,13 @@ class ShellTest(utils.TestCase):
         self.run_command('network-list')
         self.assert_called('GET', '/os-networks')
 
+    def test_network_list_fields(self):
+        output = self.run_command('network-list --fields '
+                         'vlan,project_id')
+        self.assert_called('GET', '/os-networks')
+        self.assertIn('1234', output)
+        self.assertIn('4ffc664c198e435e9853f2538fbcd7a7', output)
+
     def test_network_show(self):
         self.run_command('network-show 1')
         self.assert_called('GET', '/os-networks/1')
@@ -1757,9 +1791,32 @@ class ShellTest(utils.TestCase):
         self.run_command('network-create --fixed-range-v4 192.168.0.0/24'
                          ' --vlan=200 new_network')
         body = {'network': {'cidr': '192.168.0.0/24', 'label': 'new_network',
-                            'vlan_start': '200'}}
+                            'vlan': 200}}
         self.assert_called('POST', '/os-networks', body)
 
+    def test_network_create_vlan_start(self):
+        self.run_command('network-create --fixed-range-v4 192.168.0.0/24'
+                         ' --vlan-start=100 new_network')
+        body = {'network': {'cidr': '192.168.0.0/24', 'label': 'new_network',
+                            'vlan_start': 100}}
+        self.assert_called('POST', '/os-networks', body)
+
+    def test_network_create_extra_args(self):
+        self.run_command('network-create --fixed-range-v4 192.168.0.0/24'
+                         ' --enable-dhcp F --dhcp-server 192.168.0.2'
+                         ' --share-address T --allowed-start 192.168.0.10'
+                         ' --allowed-end 192.168.0.20 --mtu 9000 new_network')
+        body = {'network': {'cidr': '192.168.0.0/24', 'label': 'new_network',
+                            'enable_dhcp': False, 'dhcp_server': '192.168.0.2',
+                            'share_address': True, 'mtu': 9000,
+                            'allowed_start': '192.168.0.10',
+                            'allowed_end': '192.168.0.20'}}
+        self.assert_called('POST', '/os-networks', body)
+
+    def test_network_delete(self):
+        self.run_command('network-delete 1')
+        self.assert_called('DELETE', '/os-networks/1')
+
     def test_add_fixed_ip(self):
         self.run_command('add-fixed-ip sample-server 1')
         self.assert_called('POST', '/servers/1234/action',
@@ -1803,16 +1860,24 @@ class ShellTest(utils.TestCase):
                            {'evacuate': {'host': 'new_host',
                                          'onSharedStorage': False,
                                          'adminPass': 'NewAdminPass'}})
-        self.run_command('evacuate sample-server new_host')
-        self.assert_called('POST', '/servers/1234/action',
-                           {'evacuate': {'host': 'new_host',
-                                         'onSharedStorage': False}})
         self.run_command('evacuate sample-server new_host '
                          '--on-shared-storage')
         self.assert_called('POST', '/servers/1234/action',
                            {'evacuate': {'host': 'new_host',
                                          'onSharedStorage': True}})
 
+    def test_evacuate_with_no_target_host(self):
+        self.run_command('evacuate sample-server')
+        self.assert_called('POST', '/servers/1234/action',
+                           {'evacuate': {'onSharedStorage': False}})
+        self.run_command('evacuate sample-server --password NewAdminPass')
+        self.assert_called('POST', '/servers/1234/action',
+                           {'evacuate': {'onSharedStorage': False,
+                                         'adminPass': 'NewAdminPass'}})
+        self.run_command('evacuate sample-server --on-shared-storage')
+        self.assert_called('POST', '/servers/1234/action',
+                           {'evacuate': {'onSharedStorage': True}})
+
     def test_get_password(self):
         self.run_command('get-password sample-server /foo/id_rsa')
         self.assert_called('GET', '/servers/1234/os-server-password')
@@ -1836,6 +1901,13 @@ class ShellTest(utils.TestCase):
                                {'name': 'test',
                                 'description': 'FAKE_SECURITY_GROUP'}})
 
+    def test_security_group_create_without_description(self):
+        self.run_command('secgroup-create test')
+        self.assert_called('POST', '/os-security-groups',
+                           {'security_group':
+                               {'name': 'test',
+                                'description': None}})
+
     def test_security_group_update(self):
         self.run_command('secgroup-update test te FAKE_SECURITY_GROUP')
         self.assert_called('PUT', '/os-security-groups/1',
@@ -2024,10 +2096,15 @@ class ShellTest(utils.TestCase):
     def test_ssh(self, mock_system, mock_find_server):
         class FakeResources(object):
             addresses = {
-                "private": [{'version': 4, 'addr': "1.1.1.1"},
-                            {'version': 6, 'addr': "2607:f0d0:1002::4"}],
-                "public": [{'version': 4, 'addr': "2.2.2.2"},
-                           {'version': 6, 'addr': "7612:a1b2:2004::6"}]
+                "skynet": [
+                    {'version': 4, 'addr': "1.1.1.1",
+                    "OS-EXT-IPS:type": 'fixed'},
+                    {'version': 4, 'addr': "2.2.2.2",
+                    "OS-EXT-IPS:type": 'floating'},
+                    {'version': 6, 'addr': "2607:f0d0:1002::4",
+                    "OS-EXT-IPS:type": 'fixed'},
+                    {'version': 6, 'addr': "7612:a1b2:2004::6"}
+                ]
             }
         mock_find_server.return_value = FakeResources()
 
@@ -2062,6 +2139,33 @@ class ShellTest(utils.TestCase):
         mock_system.assert_called_with("ssh -6 -p22  "
                                        "root@2607:f0d0:1002::4 -1")
 
+    @mock.patch('novaclient.v1_1.shell._find_server')
+    @mock.patch('os.system')
+    def test_ssh_multinet(self, mock_system, mock_find_server):
+        class FakeResources(object):
+            addresses = {
+                "skynet": [
+                    {'version': 4, 'addr': "1.1.1.1",
+                    "OS-EXT-IPS:type": 'fixed'},
+                    {'version': 4, 'addr': "2.2.2.2"},
+                    {'version': 6, 'addr': "2607:f0d0:1002::4",
+                    "OS-EXT-IPS:type": 'fixed'}
+                ],
+                "other": [
+                    {'version': 4, 'addr': "2.3.4.5"},
+                    {'version': 6, 'addr': "7612:a1b2:2004::6"}
+                ]
+            }
+        mock_find_server.return_value = FakeResources()
+
+        self.run_command("ssh --network other server")
+        mock_system.assert_called_with("ssh -4 -p22  root@2.3.4.5 ")
+        self.run_command("ssh --ipv6 --network other server")
+        mock_system.assert_called_with("ssh -6 -p22  root@7612:a1b2:2004::6 ")
+        self.assertRaises(exceptions.ResourceNotFound,
+                          self.run_command,
+                          "ssh --ipv6 --network nonexistent server")
+
     def test_keypair_add(self):
         self.run_command('keypair-add test')
         self.assert_called('POST', '/os-keypairs',
@@ -2144,3 +2248,44 @@ class GetSecgroupTest(utils.TestCase):
                           novaclient.v1_1.shell._get_secgroup,
                           cs,
                           'group_one')
+
+
+class GetFirstEndpointTest(utils.TestCase):
+    def test_only_one_endpoint(self):
+        """If there is only one endpoint, it is returned."""
+        endpoint = {"url": "test"}
+        result = novaclient.v1_1.shell._get_first_endpoint([endpoint], "XYZ")
+        self.assertEqual(endpoint, result)
+
+    def test_multiple_endpoints(self):
+        """If there are multiple endpoints, the first one of the appropriate
+        region is returned.
+
+        """
+        endpoints = [
+            {"region": "XYZ"},
+            {"region": "ORD", "number": 1},
+            {"region": "ORD", "number": 2}
+        ]
+        result = novaclient.v1_1.shell._get_first_endpoint(endpoints, "ORD")
+        self.assertEqual(endpoints[1], result)
+
+    def test_multiple_endpoints_but_none_suitable(self):
+        """If there are multiple endpoints but none of them are suitable, an
+        exception is raised.
+
+        """
+        endpoints = [
+            {"region": "XYZ"},
+            {"region": "PQR"},
+            {"region": "STU"}
+        ]
+        self.assertRaises(LookupError,
+                          novaclient.v1_1.shell._get_first_endpoint,
+                          endpoints, "ORD")
+
+    def test_no_endpoints(self):
+        """If there are no endpoints available, an exception is raised."""
+        self.assertRaises(LookupError,
+                          novaclient.v1_1.shell._get_first_endpoint,
+                          [], "ORD")
