@@ -127,17 +127,29 @@ class ShellTest(utils.TestCase):
         self.assert_called('PUT', '/os-aggregates/1', body, pos=-2)
         self.assert_called('GET', '/os-aggregates/1', pos=-1)
 
-    def test_aggregate_set_metadata_by_id(self):
-        self.run_command('aggregate-set-metadata 1 foo=bar delete_key')
-        body = {"set_metadata": {"metadata": {"foo": "bar",
-                                              "delete_key": None}}}
-        self.assert_called('POST', '/os-aggregates/1/action', body, pos=-2)
-        self.assert_called('GET', '/os-aggregates/1', pos=-1)
+    def test_aggregate_set_metadata_add_by_id(self):
+        self.run_command('aggregate-set-metadata 3 foo=bar')
+        body = {"set_metadata": {"metadata": {"foo": "bar"}}}
+        self.assert_called('POST', '/os-aggregates/3/action', body, pos=-2)
+        self.assert_called('GET', '/os-aggregates/3', pos=-1)
+
+    def test_aggregate_set_metadata_add_duplicate_by_id(self):
+        cmd = 'aggregate-set-metadata 3 test=dup'
+        self.assertRaises(exceptions.CommandError, self.run_command, cmd)
+
+    def test_aggregate_set_metadata_delete_by_id(self):
+        self.run_command('aggregate-set-metadata 3 none_key')
+        body = {"set_metadata": {"metadata": {"none_key": None}}}
+        self.assert_called('POST', '/os-aggregates/3/action', body, pos=-2)
+        self.assert_called('GET', '/os-aggregates/3', pos=-1)
+
+    def test_aggregate_set_metadata_delete_missing_by_id(self):
+        cmd = 'aggregate-set-metadata 3 delete_key2'
+        self.assertRaises(exceptions.CommandError, self.run_command, cmd)
 
     def test_aggregate_set_metadata_by_name(self):
-        self.run_command('aggregate-set-metadata test foo=bar delete_key')
-        body = {"set_metadata": {"metadata": {"foo": "bar",
-                                              "delete_key": None}}}
+        self.run_command('aggregate-set-metadata test foo=bar')
+        body = {"set_metadata": {"metadata": {"foo": "bar"}}}
         self.assert_called('POST', '/os-aggregates/1/action', body, pos=-2)
         self.assert_called('GET', '/os-aggregates/1', pos=-1)
 
@@ -186,20 +198,6 @@ class ShellTest(utils.TestCase):
             }},
         )
 
-    def test_boot_multiple(self):
-        self.run_command('boot --flavor 1 --image 1'
-                         ' --num-instances 3 some-server')
-        self.assert_called_anytime(
-            'POST', '/servers',
-            {'server': {
-                'flavor_ref': '1',
-                'name': 'some-server',
-                'image_ref': '1',
-                'os-multiple-create:min_count': 1,
-                'os-multiple-create:max_count': 3,
-            }},
-        )
-
     def test_boot_image_with(self):
         self.run_command("boot --flavor 1"
                          " --image-with test_key=test_value some-server")
@@ -240,6 +238,7 @@ class ShellTest(utils.TestCase):
 
             mock_open.assert_called_once_with(testfile)
 
+        user_data = base64.b64encode(file_text.encode('utf-8')).decode('utf-8')
         self.assert_called_anytime(
             'POST', '/servers',
             {'server': {
@@ -248,9 +247,7 @@ class ShellTest(utils.TestCase):
                 'image_ref': '1',
                 'os-multiple-create:min_count': 1,
                 'os-multiple-create:max_count': 1,
-                'os-user-data:user_data': base64.b64encode(
-                    file_text.encode('utf-8'))
-            }},
+                'os-user-data:user_data': user_data}},
         )
 
     def test_boot_avzone(self):
@@ -344,7 +341,10 @@ class ShellTest(utils.TestCase):
                     {
                         'volume_id': 'blah',
                         'delete_on_termination': '0',
-                        'device_name': 'vda'
+                        'device_name': 'vda',
+                        'boot_index': 0,
+                        'uuid': 'blah',
+                        'source_type': ''
                     }
                 ],
                 'image_ref': '',
@@ -359,15 +359,16 @@ class ShellTest(utils.TestCase):
             'source=volume,dest=volume,device=vda,size=1,format=ext4,'
             'type=disk,shutdown=preserve some-server'
         )
+        id = ('fake-id,source=volume,dest=volume,device=vda,size=1,'
+              'format=ext4,type=disk,shutdown=preserve')
         self.assert_called_anytime(
             'POST', '/servers',
             {'server': {
                 'flavor_ref': '1',
                 'name': 'some-server',
                 'os-block-device-mapping:block_device_mapping': [
-                    {'device_name': 'id', 'volume_id':
-                        'fake-id,source=volume,dest=volume,device=vda,size=1,'
-                        'format=ext4,type=disk,shutdown=preserve'}],
+                    {'device_name': 'id', 'volume_id': id,
+                     'source_type': 'volume', 'boot_index': 0, 'uuid': id}],
                 'image_ref': '1',
                 'os-multiple-create:min_count': 1,
                 'os-multiple-create:max_count': 1,
@@ -428,7 +429,33 @@ class ShellTest(utils.TestCase):
             },
         )
 
-    def tets_boot_nics_no_value(self):
+    def test_boot_nics_ipv6(self):
+        cmd = ('boot --image 1 --flavor 1 '
+               '--nic net-id=a=c,v6-fixed-ip=2001:db9:0:1::10 some-server')
+        self.run_command(cmd)
+        self.assert_called_anytime(
+            'POST', '/servers',
+            {
+                'server': {
+                    'flavor_ref': '1',
+                    'name': 'some-server',
+                    'image_ref': '1',
+                    'os-multiple-create:min_count': 1,
+                    'os-multiple-create:max_count': 1,
+                    'networks': [
+                        {'uuid': 'a=c', 'fixed_ip': '2001:db9:0:1::10'},
+                    ],
+                },
+            },
+        )
+
+    def test_boot_nics_both_ipv4_and_ipv6(self):
+        cmd = ('boot --image 1 --flavor 1 '
+               '--nic net-id=a=c,v4-fixed-ip=10.0.0.1,'
+               'v6-fixed-ip=2001:db9:0:1::10 some-server')
+        self.assertRaises(exceptions.CommandError, self.run_command, cmd)
+
+    def test_boot_nics_no_value(self):
         cmd = ('boot --image 1 --flavor 1 '
                '--nic net-id some-server')
         self.assertRaises(exceptions.CommandError, self.run_command, cmd)
@@ -443,6 +470,11 @@ class ShellTest(utils.TestCase):
                '--nic v4-fixed-ip=10.0.0.1 some-server')
         self.assertRaises(exceptions.CommandError, self.run_command, cmd)
 
+    def test_boot_nics_netid_and_portid(self):
+        cmd = ('boot --image 1 --flavor 1 '
+               '--nic port-id=some=port,net-id=some=net some-server')
+        self.assertRaises(exceptions.CommandError, self.run_command, cmd)
+
     def test_boot_num_instances(self):
         self.run_command('boot --image 1 --flavor 1 --num-instances 3 server')
         self.assert_called_anytime(
@@ -458,11 +490,69 @@ class ShellTest(utils.TestCase):
             })
 
     def test_boot_invalid_num_instances(self):
-        cmd = 'boot --image 1 --flavor 1 --num-instances 1  server'
-        self.assertRaises(exceptions.CommandError, self.run_command, cmd)
         cmd = 'boot --image 1 --flavor 1 --num-instances 0  server'
         self.assertRaises(exceptions.CommandError, self.run_command, cmd)
 
+    def test_boot_num_instances_and_count(self):
+        cmd = 'boot --image 1 --flavor 1 --num-instances 3 --min-count 3 serv'
+        self.assertRaises(exceptions.CommandError, self.run_command, cmd)
+        cmd = 'boot --image 1 --flavor 1 --num-instances 3 --max-count 3 serv'
+        self.assertRaises(exceptions.CommandError, self.run_command, cmd)
+
+    def test_boot_min_max_count(self):
+        self.run_command('boot --image 1 --flavor 1 --max-count 3 server')
+        self.assert_called_anytime(
+            'POST', '/servers',
+            {
+                'server': {
+                    'flavor_ref': '1',
+                    'name': 'server',
+                    'image_ref': '1',
+                    'os-multiple-create:min_count': 1,
+                    'os-multiple-create:max_count': 3,
+                }
+            })
+        self.run_command('boot --image 1 --flavor 1 --min-count 3 server')
+        self.assert_called_anytime(
+            'POST', '/servers',
+            {
+                'server': {
+                    'flavor_ref': '1',
+                    'name': 'server',
+                    'image_ref': '1',
+                    'os-multiple-create:min_count': 3,
+                    'os-multiple-create:max_count': 3,
+                }
+            })
+        self.run_command('boot --image 1 --flavor 1 '
+                         '--min-count 3 --max-count 3 server')
+        self.assert_called_anytime(
+            'POST', '/servers',
+            {
+                'server': {
+                    'flavor_ref': '1',
+                    'name': 'server',
+                    'image_ref': '1',
+                    'os-multiple-create:min_count': 3,
+                    'os-multiple-create:max_count': 3,
+                }
+            })
+        self.run_command('boot --image 1 --flavor 1 '
+                         '--min-count 3 --max-count 5 server')
+        self.assert_called_anytime(
+            'POST', '/servers',
+            {
+                'server': {
+                    'flavor_ref': '1',
+                    'name': 'server',
+                    'image_ref': '1',
+                    'os-multiple-create:min_count': 3,
+                    'os-multiple-create:max_count': 5,
+                }
+            })
+        cmd = 'boot --image 1 --flavor 1 --min-count 3 --max-count 1 serv'
+        self.assertRaises(exceptions.CommandError, self.run_command, cmd)
+
     @mock.patch('novaclient.v3.shell._poll_for_status')
     def test_boot_with_poll(self, poll_method):
         self.run_command('boot --flavor 1 --image 1 some-server --poll')
@@ -480,3 +570,155 @@ class ShellTest(utils.TestCase):
         poll_method.assert_has_calls(
             [mock.call(self.shell.cs.servers.get, 1234, 'building',
                        ['active'])])
+
+    def test_boot_with_poll_to_check_VM_state_error(self):
+        self.assertRaises(exceptions.InstanceInErrorState, self.run_command,
+                          'boot --flavor 1 --image 1 some-bad-server --poll')
+
+    def test_evacuate(self):
+        self.run_command('evacuate sample-server new_host')
+        self.assert_called('POST', '/servers/1234/action',
+                           {'evacuate': {'host': 'new_host',
+                                         'on_shared_storage': False}})
+        self.run_command('evacuate sample-server new_host '
+                         '--password NewAdminPass')
+        self.assert_called('POST', '/servers/1234/action',
+                           {'evacuate': {'host': 'new_host',
+                                         'on_shared_storage': False,
+                                         'admin_password': 'NewAdminPass'}})
+        self.run_command('evacuate sample-server new_host')
+        self.assert_called('POST', '/servers/1234/action',
+                           {'evacuate': {'host': 'new_host',
+                                         'on_shared_storage': False}})
+        self.run_command('evacuate sample-server new_host '
+                         '--on-shared-storage')
+        self.assert_called('POST', '/servers/1234/action',
+                           {'evacuate': {'host': 'new_host',
+                                         'on_shared_storage': True}})
+
+    def test_evacuate_with_no_target_host(self):
+        self.run_command('evacuate sample-server')
+        self.assert_called('POST', '/servers/1234/action',
+                           {'evacuate': {'on_shared_storage': False}})
+        self.run_command('evacuate sample-server --password NewAdminPass')
+        self.assert_called('POST', '/servers/1234/action',
+                           {'evacuate': {'on_shared_storage': False,
+                                         'admin_password': 'NewAdminPass'}})
+        self.run_command('evacuate sample-server --on-shared-storage')
+        self.assert_called('POST', '/servers/1234/action',
+                           {'evacuate': {'on_shared_storage': True}})
+
+    def test_boot_named_flavor(self):
+        self.run_command(["boot", "--image", "1",
+                          "--flavor", "512 MB Server",
+                          "--max-count", "3", "server"])
+        self.assert_called('GET', '/flavors/512 MB Server', pos=0)
+        self.assert_called('GET', '/flavors?is_public=None', pos=1)
+        self.assert_called('GET', '/flavors?is_public=None', pos=2)
+        self.assert_called('GET', '/flavors/2', pos=3)
+        self.assert_called(
+            'POST', '/servers',
+            {
+                'server': {
+                    'flavor_ref': '2',
+                    'name': 'server',
+                    'image_ref': '1',
+                    'os-multiple-create:min_count': 1,
+                    'os-multiple-create:max_count': 3,
+                }
+            }, pos=4)
+
+    def test_flavor_show_by_name(self):
+        self.run_command(['flavor-show', '128 MB Server'])
+        self.assert_called('GET', '/flavors/128 MB Server', pos=0)
+        self.assert_called('GET', '/flavors?is_public=None', pos=1)
+        self.assert_called('GET', '/flavors?is_public=None', pos=2)
+        self.assert_called('GET', '/flavors/aa1', pos=3)
+        self.assert_called('GET', '/flavors/aa1/flavor-extra-specs', pos=4)
+
+    def test_flavor_show_by_name_priv(self):
+        self.run_command(['flavor-show', '512 MB Server'])
+        self.assert_called('GET', '/flavors/512 MB Server', pos=0)
+        self.assert_called('GET', '/flavors?is_public=None', pos=1)
+        self.assert_called('GET', '/flavors?is_public=None', pos=2)
+        self.assert_called('GET', '/flavors/2', pos=3)
+        self.assert_called('GET', '/flavors/2/flavor-extra-specs', pos=4)
+
+    def test_delete(self):
+        self.run_command('delete 1234')
+        self.assert_called('DELETE', '/servers/1234')
+        self.run_command('delete sample-server')
+        self.assert_called('DELETE', '/servers/1234')
+
+    def test_delete_two_with_two_existent(self):
+        self.run_command('delete 1234 5678')
+        self.assert_called('DELETE', '/servers/1234', pos=-3)
+        self.assert_called('DELETE', '/servers/5678', pos=-1)
+        self.run_command('delete sample-server sample-server2')
+        self.assert_called('GET', '/servers', pos=-6)
+        self.assert_called('GET', '/servers/1234', pos=-5)
+        self.assert_called('DELETE', '/servers/1234', pos=-4)
+        self.assert_called('GET', '/servers', pos=-3)
+        self.assert_called('GET', '/servers/5678', pos=-2)
+        self.assert_called('DELETE', '/servers/5678', pos=-1)
+
+    def test_delete_two_with_one_nonexistent(self):
+        cmd = 'delete 1234 123456789'
+        self.assertRaises(exceptions.CommandError, self.run_command, cmd)
+        self.assert_called_anytime('DELETE', '/servers/1234')
+        cmd = 'delete sample-server nonexistentserver'
+        self.assertRaises(exceptions.CommandError, self.run_command, cmd)
+        self.assert_called_anytime('DELETE', '/servers/1234')
+
+    def test_delete_one_with_one_nonexistent(self):
+        cmd = 'delete 123456789'
+        self.assertRaises(exceptions.CommandError, self.run_command, cmd)
+        cmd = 'delete nonexistent-server1'
+        self.assertRaises(exceptions.CommandError, self.run_command, cmd)
+
+    def test_delete_two_with_two_nonexistent(self):
+        cmd = 'delete 123456789 987654321'
+        self.assertRaises(exceptions.CommandError, self.run_command, cmd)
+        cmd = 'delete nonexistent-server1 nonexistent-server2'
+        self.assertRaises(exceptions.CommandError, self.run_command, cmd)
+
+
+class GetFirstEndpointTest(utils.TestCase):
+    def test_only_one_endpoint(self):
+        """If there is only one endpoint, it is returned."""
+        endpoint = {"url": "test"}
+        result = novaclient.v3.shell._get_first_endpoint([endpoint], "XYZ")
+        self.assertEqual(endpoint, result)
+
+    def test_multiple_endpoints(self):
+        """If there are multiple endpoints, the first one of the appropriate
+        region is returned.
+
+        """
+        endpoints = [
+            {"region": "XYZ"},
+            {"region": "ORD", "number": 1},
+            {"region": "ORD", "number": 2}
+        ]
+        result = novaclient.v3.shell._get_first_endpoint(endpoints, "ORD")
+        self.assertEqual(endpoints[1], result)
+
+    def test_multiple_endpoints_but_none_suitable(self):
+        """If there are multiple endpoints but none of them are suitable, an
+        exception is raised.
+
+        """
+        endpoints = [
+            {"region": "XYZ"},
+            {"region": "PQR"},
+            {"region": "STU"}
+        ]
+        self.assertRaises(LookupError,
+                          novaclient.v3.shell._get_first_endpoint,
+                          endpoints, "ORD")
+
+    def test_no_endpoints(self):
+        """If there are no endpoints available, an exception is raised."""
+        self.assertRaises(LookupError,
+                          novaclient.v3.shell._get_first_endpoint,
+                          [], "ORD")
