@@ -35,12 +35,14 @@ from novaclient.v1_1 import limits
 from novaclient.v1_1 import networks
 from novaclient.v1_1 import quota_classes
 from novaclient.v1_1 import quotas
+from novaclient.v1_1 import security_group_default_rules
 from novaclient.v1_1 import security_group_rules
 from novaclient.v1_1 import security_groups
 from novaclient.v1_1 import server_groups
 from novaclient.v1_1 import servers
 from novaclient.v1_1 import services
 from novaclient.v1_1 import usage
+from novaclient.v1_1 import versions
 from novaclient.v1_1 import virtual_interfaces
 from novaclient.v1_1 import volume_snapshots
 from novaclient.v1_1 import volume_types
@@ -55,6 +57,19 @@ class Client(object):
 
         >>> client = Client(USERNAME, PASSWORD, PROJECT_ID, AUTH_URL)
 
+    Or, alternatively, you can create a client instance using the
+    keystoneclient.session API::
+
+        >>> from keystoneclient.auth.identity import v2
+        >>> from keystoneclient import session
+        >>> from novaclient.client import Client
+        >>> auth = v2.Password(auth_url=AUTH_URL,
+                               username=USERNAME,
+                               password=PASSWORD,
+                               tenant_name=PROJECT_ID)
+        >>> sess = session.Session(auth=auth)
+        >>> nova = client.Client(VERSION, session=sess)
+
     Then call methods on its managers::
 
         >>> client.servers.list()
@@ -107,6 +122,7 @@ class Client(object):
         self.images = images.ImageManager(self)
         self.limits = limits.LimitsManager(self)
         self.servers = servers.ServerManager(self)
+        self.versions = versions.VersionManager(self)
 
         # extensions
         self.agents = agents.AgentsManager(self)
@@ -127,6 +143,8 @@ class Client(object):
         self.security_groups = security_groups.SecurityGroupManager(self)
         self.security_group_rules = \
             security_group_rules.SecurityGroupRuleManager(self)
+        self.security_group_default_rules = \
+            security_group_default_rules.SecurityGroupDefaultRuleManager(self)
         self.usage = usage.UsageManager(self)
         self.virtual_interfaces = \
             virtual_interfaces.VirtualInterfaceManager(self)
