@@ -21,13 +21,15 @@ Server interface.
 
 import base64
 
+from oslo.utils import encodeutils
 import six
 from six.moves.urllib import parse
 
 from novaclient import base
 from novaclient import crypto
-from novaclient.openstack.common import strutils
-from novaclient.v1_1.security_groups import SecurityGroup
+from novaclient.openstack.common.gettextutils import _
+from novaclient.v1_1 import security_groups
+
 
 REBOOT_SOFT, REBOOT_HARD = 'SOFT', 'HARD'
 
@@ -36,7 +38,7 @@ class Server(base.Resource):
     HUMAN_ID = True
 
     def __repr__(self):
-        return "<Server: %s>" % self.name
+        return '<Server: %s>' % getattr(self, 'name', 'unknown-name')
 
     def delete(self):
         """
@@ -363,7 +365,7 @@ class Server(base.Resource):
         """
         return self.manager.list_security_group(self)
 
-    def evacuate(self, host, on_shared_storage, password=None):
+    def evacuate(self, host=None, on_shared_storage=True, password=None):
         """
         Evacuate an instance from failed host to specified host.
 
@@ -434,8 +436,8 @@ class ServerManager(base.BootingManagerWithFind):
                       connected networks, fixed ips, etc.
         :param scheduler_hints: (optional extension) arbitrary key-value pairs
                               specified by the client to help boot an instance.
-        :param config_drive: (optional extension) value for config drive
-                            either boolean, or volume-id
+        :param config_drive: (optional extension) If True, enable config drive
+                             on the server.
         :param admin_pass: admin password for the server.
         :param disk_config: (optional extension) control how the disk is
                             partitioned when the server is created.
@@ -452,9 +454,10 @@ class ServerManager(base.BootingManagerWithFind):
             if six.PY3:
                 userdata = userdata.encode("utf-8")
             else:
-                userdata = strutils.safe_encode(userdata)
+                userdata = encodeutils.safe_encode(userdata)
 
-            body["server"]["user_data"] = base64.b64encode(userdata)
+            userdata_b64 = base64.b64encode(userdata).decode('utf-8')
+            body["server"]["user_data"] = userdata_b64
         if meta:
             body["server"]["metadata"] = meta
         if reservation_id:
@@ -490,9 +493,11 @@ class ServerManager(base.BootingManagerWithFind):
                     data = file_or_string.read()
                 else:
                     data = file_or_string
+
+                cont = base64.b64encode(data.encode('utf-8')).decode('utf-8')
                 personality.append({
                     'path': filepath,
-                    'contents': base64.b64encode(data.encode('utf-8')),
+                    'contents': cont,
                 })
 
         if availability_zone:
@@ -520,8 +525,15 @@ class ServerManager(base.BootingManagerWithFind):
                 # if value is empty string, do not send value in body
                 if nic_info.get('net-id'):
                     net_data['uuid'] = nic_info['net-id']
-                if nic_info.get('v4-fixed-ip'):
+                if (nic_info.get('v4-fixed-ip') and
+                    nic_info.get('v6-fixed-ip')):
+                    raise base.exceptions.CommandError(_(
+                        "Only one of 'v4-fixed-ip' and 'v6-fixed-ip' may be"
+                        " provided."))
+                elif nic_info.get('v4-fixed-ip'):
                     net_data['fixed_ip'] = nic_info['v4-fixed-ip']
+                elif nic_info.get('v6-fixed-ip'):
+                    net_data['fixed_ip'] = nic_info['v6-fixed-ip']
                 if nic_info.get('port-id'):
                     net_data['port'] = nic_info['port-id']
                 all_net_data.append(net_data)
@@ -911,7 +923,8 @@ class ServerManager(base.BootingManagerWithFind):
         self._action('reboot', server, {'type': reboot_type})
 
     def rebuild(self, server, image, password=None, disk_config=None,
-                preserve_ephemeral=False, **kwargs):
+                preserve_ephemeral=False, name=None, meta=None, files=None,
+                **kwargs):
         """
         Rebuild -- shut down and then re-image -- a server.
 
@@ -922,6 +935,15 @@ class ServerManager(base.BootingManagerWithFind):
                             Valid values are 'AUTO' or 'MANUAL'
         :param preserve_ephemeral: If True, request that any ephemeral device
             be preserved when rebuilding the instance. Defaults to False.
+        :param name: Something to name the server.
+        :param meta: A dict of arbitrary key/value metadata to store for this
+                     server. A maximum of five entries is allowed, and both
+                     keys and values must be 255 characters or less.
+        :param files: A dict of files to overwrite on the server upon boot.
+                      Keys are file names (i.e. ``/etc/passwd``) and values
+                      are the file contents (either as a string or as a
+                      file-like object). A maximum of five entries is allowed,
+                      and each file must be 10k or less.
         """
         body = {'imageRef': base.getid(image)}
         if password is not None:
@@ -930,6 +952,24 @@ class ServerManager(base.BootingManagerWithFind):
             body['OS-DCF:diskConfig'] = disk_config
         if preserve_ephemeral is not False:
             body['preserve_ephemeral'] = True
+        if name is not None:
+            body['name'] = name
+        if meta:
+            body['metadata'] = meta
+        if files:
+            personality = body['personality'] = []
+            for filepath, file_or_string in sorted(files.items(),
+                                                   key=lambda x: x[0]):
+                if hasattr(file_or_string, 'read'):
+                    data = file_or_string.read()
+                else:
+                    data = file_or_string
+
+                cont = base64.b64encode(data.encode('utf-8')).decode('utf-8')
+                personality.append({
+                    'path': filepath,
+                    'contents': cont,
+                })
 
         _resp, body = self._action('rebuild', server, body, **kwargs)
         return Server(self, body['server'])
@@ -1108,9 +1148,11 @@ class ServerManager(base.BootingManagerWithFind):
 
         """
         return self._list('/servers/%s/os-security-groups' %
-                          base.getid(server), 'security_groups', SecurityGroup)
+                          base.getid(server), 'security_groups',
+                          security_groups.SecurityGroup)
 
-    def evacuate(self, server, host, on_shared_storage, password=None):
+    def evacuate(self, server, host=None, on_shared_storage=True,
+                 password=None):
         """
         Evacuate a server instance.
 
@@ -1120,10 +1162,10 @@ class ServerManager(base.BootingManagerWithFind):
                         on shared storage
         :param password: string to set as password on the evacuated server.
         """
-        body = {
-                'host': host,
-                'onSharedStorage': on_shared_storage,
-                }
+
+        body = {'onSharedStorage': on_shared_storage}
+        if host is not None:
+            body['host'] = host
 
         if password is not None:
             body['adminPass'] = password
