@@ -21,12 +21,13 @@ Server interface.
 
 import base64
 
+from oslo.utils import encodeutils
 import six
 from six.moves.urllib import parse
 
 from novaclient import base
 from novaclient import crypto
-from novaclient.openstack.common import strutils
+from novaclient.openstack.common.gettextutils import _
 
 REBOOT_SOFT, REBOOT_HARD = 'SOFT', 'HARD'
 
@@ -319,7 +320,7 @@ class Server(base.Resource):
         """
         self.manager.reset_network(self)
 
-    def evacuate(self, host, on_shared_storage, password=None):
+    def evacuate(self, host=None, on_shared_storage=True, password=None):
         """
         Evacuate an instance from failed host to specified host.
 
@@ -401,15 +402,15 @@ class ServerManager(base.BootingManagerWithFind):
             if six.PY3:
                 userdata = userdata.encode("utf-8")
             else:
-                userdata = strutils.safe_encode(userdata)
+                userdata = encodeutils.safe_encode(userdata)
 
-            body["server"][
-                "os-user-data:user_data"] = base64.b64encode(userdata)
+            data = base64.b64encode(userdata).decode('utf-8')
+            body["server"]["os-user-data:user_data"] = data
         if meta:
             body["server"]["metadata"] = meta
         if reservation_id:
             body["server"][
-                "os-multiple-create:reservation_id"] = reservation_id
+                "os-multiple-create:return_reservation_id"] = reservation_id
         if key_name:
             body["server"]["key_name"] = key_name
         if scheduler_hints:
@@ -457,8 +458,15 @@ class ServerManager(base.BootingManagerWithFind):
                 # if value is empty string, do not send value in body
                 if nic_info.get('net-id'):
                     net_data['uuid'] = nic_info['net-id']
-                if nic_info.get('v4-fixed-ip'):
+                if (nic_info.get('v4-fixed-ip') and
+                    nic_info.get('v6-fixed-ip')):
+                    raise base.exceptions.CommandError(_(
+                        "Only one of 'v4-fixed-ip' and 'v6-fixed-ip' may be"
+                        " provided."))
+                elif nic_info.get('v4-fixed-ip'):
                     net_data['fixed_ip'] = nic_info['v4-fixed-ip']
+                elif nic_info.get('v6-fixed-ip'):
+                    net_data['fixed_ip'] = nic_info['v6-fixed-ip']
                 if nic_info.get('port-id'):
                     net_data['port'] = nic_info['port-id']
                 all_net_data.append(net_data)
@@ -894,6 +902,11 @@ class ServerManager(base.BootingManagerWithFind):
                         you would like to retrieve.
         :param length: The number of tail loglines you would like to retrieve.
         """
+        if length is None:
+            # NOTE: On v3 get_console_output API, -1 means an unlimited length.
+            # Here translates None, which means an unlimited in the internal
+            # implementation, to -1.
+            length = -1
         return self._action('get_console_output',
                             server, {'length': length})[1]['output']
 
@@ -937,7 +950,8 @@ class ServerManager(base.BootingManagerWithFind):
         """
         self._action('reset_network', server)
 
-    def evacuate(self, server, host, on_shared_storage, password=None):
+    def evacuate(self, server, host=None,
+                 on_shared_storage=True, password=None):
         """
         Evacuate a server instance.
 
@@ -947,10 +961,9 @@ class ServerManager(base.BootingManagerWithFind):
                         on shared storage
         :param password: string to set as password on the evacuated server.
         """
-        body = {
-                'host': host,
-                'on_shared_storage': on_shared_storage,
-                }
+        body = {'on_shared_storage': on_shared_storage}
+        if host is not None:
+            body['host'] = host
 
         if password is not None:
             body['admin_password'] = password
